Module timely::communication [−][src]
Re-export of the timely_communication crate.
Modules
| allocator | Types and traits for the allocation of channels. |
| buzzer | A type that can unpark specific threads. |
| initialize | Initialization logic for a generic instance of the |
| logging | Configuration and events for communication logging. |
| message | Types wrapping typed data. |
| networking | Networking code for sending and receiving fixed size |
Structs
| Message | A wrapped message which may be either typed or binary data. |
| WorkerGuards | Maintains |
Enums
| Allocator | Enumerates known implementors of |
| Config | Possible configurations for the communication infrastructure. |
Traits
| Allocate | A type capable of allocating channels. |
| Data | A composite trait for types that may be used with channels. |
| Pull | Pulling elements of type |
| Push | Pushing elements of type |
Functions
| initialize | Initializes communication and executes a distributed computation. |
| initialize_from | Initializes computation and runs a distributed computation. |