pub struct TcpBuilder { /* private fields */ }Expand description
Builds an instance of a TcpAllocator.
Builders are required because some of the state in a TcpAllocator cannot be sent between
threads (specifically, the Rc<RefCell<_>> local channels). So, we must package up the state
shared between threads here, and then provide a method that will instantiate the non-movable
members once in the destination thread.
Implementations
Sourceimpl TcpBuilder
impl TcpBuilder
Sourcepub fn build(self) -> TcpAllocator
pub fn build(self) -> TcpAllocator
Builds a TcpAllocator, instantiating Rc<RefCell<_>> elements.