MessageHeader

Struct MessageHeader 

pub struct MessageHeader {
    pub channel: usize,
    pub source: usize,
    pub target_lower: usize,
    pub target_upper: usize,
    pub length: usize,
    pub seqno: usize,
}
Framing data for each Vec<u8> transmission, indicating a typed channel, the source and destination workers, and the length in bytes.

Fields

channel: usize

index of channel.

source: usize

index of worker sending message.

target_lower: usize

lower bound of index of worker receiving message.

target_upper: usize

upper bound of index of worker receiving message.

This is often self.target_lower + 1 for point to point messages, but can be larger for broadcast messages.

length: usize

number of bytes in message.

seqno: usize

sequence number.

Implementations

Source

impl MessageHeader

pub fn try_read(bytes: &[u8]) -> Option<MessageHeader>

Returns a header when there is enough supporting data

pub fn write_to<W>(&self, writer: &mut W) -> Result<(), Error>
where W: Write,

Writes the header as binary data.

pub fn required_bytes(&self) -> usize

The number of bytes required for the header and data.

pub fn header_bytes(&self) -> usize

The number of bytes required for the header.