Bytesable

Trait Bytesable 

pub trait Bytesable {
    // Required methods
    fn from_bytes(bytes: Bytes) -> Self;
    fn length_in_bytes(&self) -> usize;
    fn into_bytes<W>(&self, writer: &mut W)
       where W: Write;
}
A type that can be serialized and deserialized through Bytes.

Required Methods

fn from_bytes(bytes: Bytes) -> Self

Wrap bytes as Self.

fn length_in_bytes(&self) -> usize

The number of bytes required to serialize the data.

fn into_bytes<W>(&self, writer: &mut W)
where W: Write,

Writes the binary representation into writer.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<T, C> Bytesable for Message<T, C>
where T: Serialize + for<'a> Deserialize<'a>, C: ContainerBytes,

Source

impl<T: Data> Bytesable for Bincode<T>