pub struct NoopBuilder<C> { /* private fields */ }Expand description
A container builder that only accepts containers, and produces them as output.
This exists for operators that have containers ready to go, and haven’t the need for a builder.