Module dataflow

Module dataflow 

Abstractions for timely dataflow programming.

Timely dataflow programs are constructed by manipulating Stream objects, most often using pre-defined operators that implement known patterns.

Examples

use timely::dataflow::operators::{ToStream, Inspect};

timely::example(|scope| {
    (0..10).to_stream(scope)
           .container::<Vec<_>>()
           .inspect(|x| println!("seen: {:?}", x));
});

Re-exports

pub use self::stream::Stream;
pub use self::stream::StreamVec;
pub use self::scope::Scope;
pub use self::scope::OperatorSlot;
pub use self::operators::core::input::Handle as InputHandle;
pub use self::operators::vec::input::Handle as InputHandleVec;
pub use self::operators::probe::Handle as ProbeHandle;

Modules

channels
Structured communication between timely dataflow operators.
operators
Extension traits for Stream and StreamVec implementing various operators.
scope
A dataflow scope, used to build dataflow graphs.
stream
A handle to a typed stream of timely data.