Module buffer

Module buffer 

Buffering and session mechanisms to provide the appearance of record-at-a-time sending, with the performance of batched sends.

Structs

AutoflushSessionCore
A session which will flush itself when dropped.
BufferCore
Buffers data sent at the same time, for efficient communication.
Session
An output session for sending records at a specified time.

Type Aliases

AutoflushSession
Auto-flush session specialized to vector-based containers.
Buffer
A buffer specialized to vector-based containers.