TeeHelper

Struct TeeHelper 

pub struct TeeHelper<T, C> { /* private fields */ }
The subscribe half of a shared destination for pushing at.

Cloning a TeeHelper will upgrade it, teaching the shared list how to clone containers.

Implementations

Source

impl<T: 'static, C: 'static> TeeHelper<T, C>

pub fn upgrade(&self)
where T: Clone, C: Clone,

Upgrades the shared list to one that supports cloning.

This method “teaches” the Tee how to clone containers, which enables adding multiple pushers. It introduces the cost of one additional virtual call through a boxed trait, so one should not upgrade for no reason.

pub fn add_pusher<P: Push<Message<T, C>> + 'static>(self, pusher: P)

Adds a new Push implementor to the list of recipients shared with a Stream.