Replay

Trait Replay 

pub trait Replay<T: Timestamp, C>: Sized {
    // Required method
    fn replay_core<'scope>(
        self,
        scope: Scope<'scope, T>,
        period: Option<Duration>,
    ) -> Stream<'scope, T, C>;

    // Provided method
    fn replay_into<'scope>(
        self,
        scope: Scope<'scope, T>,
    ) -> Stream<'scope, T, C> { ... }
}
Replay a capture stream into a scope with the same timestamp.

Required Methods

fn replay_core<'scope>( self, scope: Scope<'scope, T>, period: Option<Duration>, ) -> Stream<'scope, T, C>

Replays self into the provided scope, as a Stream<'scope, T, C>.

The period argument allows the specification of a re-activation period, where the operator will re-activate itself every so often. The None argument instructs the operator not to re-activate itself.

Provided Methods

fn replay_into<'scope>(self, scope: Scope<'scope, T>) -> Stream<'scope, T, C>

Replays self into the provided scope, as a Stream<'scope, T, C>.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors

Source

impl<T: Timestamp, C: Container + Clone, I> Replay<T, C> for I
where I: IntoIterator, <I as IntoIterator>::Item: EventIterator<T, C> + 'static,