Enter

Trait Enter 

pub trait Enter<G: Scope, T: Timestamp + Refines<G::Timestamp>, C> {
    // Required method
    fn enter<'a>(self, _: &Child<'a, G, T>) -> Stream<Child<'a, G, T>, C>;
}
Extension trait to move a Stream into a child of its current Scope.

Required Methods

fn enter<'a>(self, _: &Child<'a, G, T>) -> Stream<Child<'a, G, T>, C>

Moves the Stream argument into a child of its current Scope.

Examples
use timely::dataflow::scopes::Scope;
use timely::dataflow::operators::{Enter, Leave, ToStream};

timely::example(|outer| {
    let stream = (0..9).to_stream(outer).container::<Vec<_>>();
    let output = outer.region(|inner| {
        stream.enter(inner).leave()
    });
});

Implementors

Source

impl<G: Scope, T: Timestamp + Refines<G::Timestamp>, C: Container> Enter<G, T, C> for Stream<G, C>