ConnectLoop

Trait ConnectLoop 

pub trait ConnectLoop<G: Scope, C: Container> {
    // Required method
    fn connect_loop(self, handle: Handle<G, C>);
}
Connect a Stream to the input of a loop variable.

Required Methods

fn connect_loop(self, handle: Handle<G, C>)

Connect a Stream to be the input of a loop variable.

Examples
use timely::dataflow::Scope;
use timely::dataflow::operators::{Feedback, ConnectLoop, ToStream, Concat, Inspect};
use timely::dataflow::operators::vec::BranchWhen;

timely::example(|scope| {
    // circulate 0..10 for 100 iterations.
    let (handle, cycle) = scope.feedback(1);
    (0..10).to_stream(scope)
           .container::<Vec<_>>()
           .concat(cycle)
           .inspect(|x| println!("seen: {:?}", x))
           .branch_when(|t| t < &100).1
           .connect_loop(handle);
});

Implementors

Source

impl<G: Scope, C: Container> ConnectLoop<G, C> for Stream<G, C>