pub trait SharedStream<S: Scope, C> {
// Required method
fn shared(self) -> Stream<S, Rc<C>>;
}Expand description
Convert a stream into a stream of shared containers
Required Methods
Convert a stream into a stream of shared data
Examples
use timely::dataflow::operators::{ToStream, InspectCore};
use timely::dataflow::operators::rc::SharedStream;
timely::example(|scope| {
(0..10).to_stream(scope)
.container::<Vec<_>>()
.shared()
.inspect_container(|x| println!("seen: {:?}", x));
});