pub type StreamVec<S, D> = Stream<S, Vec<D>>;Expand description
A stream batching data in owning vectors.
Aliased Type
pub struct StreamVec<S, D> { /* private fields */ }Trait Implementations
Sourceimpl<S: Scope<Timestamp: Hash>, K: ExchangeData + Clone + Hash + Eq, V: ExchangeData> Aggregate<S, K, V> for StreamVec<S, (K, V)>
impl<S: Scope<Timestamp: Hash>, K: ExchangeData + Clone + Hash + Eq, V: ExchangeData> Aggregate<S, K, V> for StreamVec<S, (K, V)>
Sourceimpl<S: Scope, D: 'static> Branch<S, D> for StreamVec<S, D>
impl<S: Scope, D: 'static> Branch<S, D> for StreamVec<S, D>
Sourcefn branch(
self,
condition: impl Fn(&S::Timestamp, &D) -> bool + 'static,
) -> (StreamVec<S, D>, StreamVec<S, D>)
fn branch( self, condition: impl Fn(&S::Timestamp, &D) -> bool + 'static, ) -> (StreamVec<S, D>, StreamVec<S, D>)
Takes one input stream and splits it into two output streams.
For each record, the supplied closure is called with a reference to
the data and its time. If it returns
true, the record will be sent
to the second returned stream, otherwise it will be sent to the first. Read moreSourceimpl<G: Scope<Timestamp: Hash>, D: 'static> Delay<G, D> for StreamVec<G, D>
impl<G: Scope<Timestamp: Hash>, D: 'static> Delay<G, D> for StreamVec<G, D>
Sourcefn delay<L: FnMut(&D, &G::Timestamp) -> G::Timestamp + 'static>(
self,
func: L,
) -> Self
fn delay<L: FnMut(&D, &G::Timestamp) -> G::Timestamp + 'static>( self, func: L, ) -> Self
Advances the timestamp of records using a supplied function. Read more
Sourcefn delay_total<L: FnMut(&D, &G::Timestamp) -> G::Timestamp + 'static>(
self,
func: L,
) -> Selfwhere
G::Timestamp: TotalOrder,
fn delay_total<L: FnMut(&D, &G::Timestamp) -> G::Timestamp + 'static>(
self,
func: L,
) -> Selfwhere
G::Timestamp: TotalOrder,
Advances the timestamp of records using a supplied function. Read more
Sourceimpl<S: Scope, D: 'static> Map<S, D> for StreamVec<S, D>
impl<S: Scope, D: 'static> Map<S, D> for StreamVec<S, D>
Sourcefn map_in_place<L: FnMut(&mut D) + 'static>(self, logic: L) -> StreamVec<S, D>
fn map_in_place<L: FnMut(&mut D) + 'static>(self, logic: L) -> StreamVec<S, D>
Updates each element of the stream and yields the element, re-using memory where possible. Read more