Module operate

Module operate 

Methods which describe an operators topology, and the progress it makes.

Structs

SharedProgress
Progress information shared between parent and child.

Traits

Operate
Methods for describing an operators topology, and the progress it makes.