pub enum FrontierInterest {
Never,
IfCapability,
Always,
}Expand description
The ways in which an operator can express interest in activation when an input frontier changes.
Variants
Never
Never interested in frontier changes, as for example the map() and filter() operators.
IfCapability
Interested when the operator holds capabilities.
Always
Always interested in frontier changes, as for example the probe() and capture() operators.