Struct timely::scheduling::activate::Activations[][src]

pub struct Activations { /* fields omitted */ }

Allocation-free activation tracker.

Implementations

impl Activations[src]

pub fn new(timer: Instant) -> Self[src]

Creates a new activation tracker.

pub fn activate(&mut self, path: &[usize])[src]

Activates the task addressed by path.

pub fn activate_after(&mut self, path: &[usize], delay: Duration)[src]

Schedules a future activation for the task addressed by path.

pub fn advance(&mut self)[src]

Discards the current active set and presents the next active set.

pub fn map_active(&self, logic: impl Fn(&[usize]))[src]

Maps a function across activated paths.

pub fn for_extensions(&self, path: &[usize], action: impl FnMut(usize))[src]

Sets as active any symbols that follow path.

pub fn sync(&self) -> SyncActivations[src]

Constructs a thread-safe SyncActivations handle to this activator.

pub fn empty_for(&self) -> Option<Duration>[src]

Time until next scheduled event.

This method should be used before putting a worker thread to sleep, as it indicates the amount of time before the thread should be unparked for the next scheduled activation.