Schedule

Trait Schedule 

pub trait Schedule {
    // Required methods
    fn name(&self) -> &str;
    fn path(&self) -> &[usize];
    fn schedule(&mut self) -> bool;
}
A type that can be scheduled.

Required Methods

fn name(&self) -> &str

A descriptive name for the operator

fn path(&self) -> &[usize]

An address identifying the operator.

fn schedule(&mut self) -> bool

Schedules the operator, receives “cannot terminate” boolean.

The return value indicates whether self has outstanding work and would be upset if the computation terminated.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<TOuter, TInner> Schedule for Subgraph<TOuter, TInner>
where TOuter: Timestamp, TInner: Timestamp + Refines<TOuter>,