pub struct Barrier { /* private fields */ }Expand description
A re-usable barrier synchronization mechanism.
Implementations
Sourceimpl Barrier
impl Barrier
Sourcepub fn wait(&mut self)
pub fn wait(&mut self)
Blocks until all other workers have reached this barrier.
This method does not block dataflow execution, which continues to execute while we await the arrival of the other workers.