pub trait ExchangeData: Data + Data { }Expand description
A composite trait for types usable on exchange channels in timely dataflow.
The ExchangeData trait extends Data with any requirements imposed by the timely_communication
Data trait, which describes requirements for communication along channels.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".