The root of each single-threaded worker.
Structs
- Worker configuration.
- A
Workeris the entry point to a timely dataflow computation. It wraps aAllocate, and has a list of dataflows that it manages.
Enums
- Different ways in which timely’s progress tracking can work.
Traits
- Methods provided by the root Worker.