pub enum AllocatorBuilder {
Thread(ThreadBuilder),
Process(ProcessBuilder),
Tcp(TcpBuilder),
}Expand description
Enumerations of constructable implementors of Allocate.
The builder variants are meant to be Send, so that they can be moved across threads,
whereas the allocator they construct may not. As an example, the binary Process type
contains Rc wrapped state, and so cannot itself be moved across threads.
Variants
Thread(ThreadBuilder)
Builder for the Thread allocator.
Process(ProcessBuilder)
Builder for a Process allocator (in either of two flavors, see ProcessBuilder).
Tcp(TcpBuilder)
Builder for the Tcp (inter-process) allocator.