pub struct Session<'a, T, C, P: Push<BundleCore<T, C>> + 'a>{ /* private fields */ }Expand description
An output session for sending records at a specified time.
The Session struct provides the user-facing interface to an operator output, namely
the Buffer type. A Session wraps a session of output at a specified time, and
avoids what would otherwise be a constant cost of checking timestamp equality.
Implementations
Sourceimpl<'a, T, C, P: Push<BundleCore<T, C>> + 'a> Session<'a, T, C, P>
impl<'a, T, C, P: Push<BundleCore<T, C>> + 'a> Session<'a, T, C, P>
Sourcepub fn give_container(&mut self, container: &mut C)
pub fn give_container(&mut self, container: &mut C)
Provide a container at the time specified by the Session.
Sourceimpl<'a, T, D, P: Push<BundleCore<T, Vec<D>>> + 'a> Session<'a, T, Vec<D>, P>
impl<'a, T, D, P: Push<BundleCore<T, Vec<D>>> + 'a> Session<'a, T, Vec<D>, P>
Sourcepub fn give_iterator<I: Iterator<Item = D>>(&mut self, iter: I)
pub fn give_iterator<I: Iterator<Item = D>>(&mut self, iter: I)
Provides an iterator of records at the time specified by the Session.
Sourcepub fn give_vec(&mut self, message: &mut Vec<D>)
pub fn give_vec(&mut self, message: &mut Vec<D>)
Provides a fully formed Content<D> message for senders which can use this type.
The Content type is the backing memory for communication in timely, and it can
often be more efficient to reuse this memory rather than have timely allocate
new backing memory.