pub trait Replay<T: Timestamp, C>: Sized {
// Required method
fn replay_core<'scope>(
self,
scope: Scope<'scope, T>,
period: Option<Duration>,
) -> Stream<'scope, T, C>;
// Provided method
fn replay_into<'scope>(
self,
scope: Scope<'scope, T>,
) -> Stream<'scope, T, C> { ... }
}Expand description
Replay a capture stream into a scope with the same timestamp.
Required Methods
Sourcefn replay_core<'scope>(
self,
scope: Scope<'scope, T>,
period: Option<Duration>,
) -> Stream<'scope, T, C>
fn replay_core<'scope>( self, scope: Scope<'scope, T>, period: Option<Duration>, ) -> Stream<'scope, T, C>
Replays self into the provided scope, as a Stream<'scope, T, C>.
The period argument allows the specification of a re-activation period, where the operator
will re-activate itself every so often. The None argument instructs the operator not to
re-activate itself.
Provided Methods
Sourcefn replay_into<'scope>(self, scope: Scope<'scope, T>) -> Stream<'scope, T, C>
fn replay_into<'scope>(self, scope: Scope<'scope, T>) -> Stream<'scope, T, C>
Replays self into the provided scope, as a Stream<'scope, T, C>.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.