pub trait Leave<G: Scope, C> {
// Required method
fn leave(self) -> Stream<G, C>;
}Expand description
Extension trait to move a Stream to the parent of its current Scope.
Required Methods
Sourcefn leave(self) -> Stream<G, C>
fn leave(self) -> Stream<G, C>
Moves a Stream to the parent of its current Scope.
Examples
use timely::dataflow::scopes::Scope;
use timely::dataflow::operators::{Enter, Leave, ToStream};
timely::example(|outer| {
let stream = (0..9).to_stream(outer).container::<Vec<_>>();
let output = outer.region(|inner| {
stream.enter(inner).leave()
});
});