Module timely::logging [−][src]
Traits, implementations, and macros related to logging timely events.
Structs
| ApplicationEvent | Application-defined code start or stop |
| BatchLogger | Logs events as a timely stream, with progress statements. |
| ChannelsEvent | The creation of a channel between operators. |
| CommChannelsEvent | Event on a communication channel |
| GuardedMessageEvent | Application-defined code start or stop |
| GuardedProgressEvent | Application-defined code start or stop |
| InputEvent | Input logic start/stop |
| MessagesEvent | Message send or receive event |
| OperatesEvent | The creation of an |
| PushProgressEvent | External progress pushed onto an operator |
| ScheduleEvent | Operator start or stop. |
| ShutdownEvent | Operator shutdown. |
| TimelyProgressEvent | Send or receive of progress information. |
| TimelySetup | Identifier of the worker that generated a log line |
Enums
| CommChannelKind | Kind of communication channel |
| ParkEvent | Records the starting and stopping of an operator. |
| StartStop | Records the starting and stopping of an operator. |
| TimelyEvent | An event in a timely worker |
Traits
| ProgressEventTimestamp | Encapsulates Any and Debug for dynamically typed timestamps in logs |
| ProgressEventTimestampVec | A vector of progress updates in logs |
Type Definitions
| Logger | Logger type for worker-local logging. |
| TimelyLogger | Logger for timely dataflow system events. |
| TimelyProgressLogger | Logger for timely dataflow progress events (the “timely/progress” log stream). |
| WorkerIdentifier | Type alias for logging timely events. |