Struct timely::logging::BatchLogger [−][src]
Logs events as a timely stream, with progress statements.
Implementations
impl<T, E, P> BatchLogger<T, E, P> where
P: EventPusher<Duration, (Duration, E, T)>, [src]
P: EventPusher<Duration, (Duration, E, T)>,
pub fn new(event_pusher: P) -> Self[src]
Creates a new batch logger.
pub fn publish_batch(
&mut self,
time: &Duration,
data: &mut Vec<(Duration, E, T)>
)[src]
&mut self,
time: &Duration,
data: &mut Vec<(Duration, E, T)>
)
Publishes a batch of logged events and advances the capability.