[−][src]Module timely::progress
Progress tracking mechanisms to support notification in timely dataflow
Re-exports
pub use self::operate::Operate; |
pub use self::subgraph::Subgraph; |
pub use self::subgraph::SubgraphBuilder; |
pub use self::timestamp::Timestamp; |
pub use self::timestamp::PathSummary; |
pub use self::change_batch::ChangeBatch; |
pub use self::frontier::Antichain; |
Modules
| broadcast | Broadcasts progress information among workers. |
| change_batch | A collection of updates of the form |
| frontier | Tracks minimal sets of mutually incomparable elements of a partial order. |
| operate | Methods which describe an operators topology, and the progress it makes. |
| reachability | Manages pointstamp reachability within a timely dataflow graph. |
| subgraph | A dataflow subgraph |
| timestamp | A partially ordered measure of progress at each timely dataflow location. |
Structs
| Location | A timely dataflow location. |
| Source | Names a source of a data stream. |
| Target | Names a target of a data stream. |
Enums
| Port | An operator port. |