Struct timely::progress::count_map::CountMap [] [src]

pub struct CountMap<T> { /* fields omitted */ }

Represents a map from T to i64 with values mapping to zero removed.

The implementation is currently a Vec<(T, i64)> as the use cases are currently for small buffers of updates rather than large storage. This could change as needs evolve.

Methods

impl<T: Eq + Clone> CountMap<T>
[src]

Adds val to the value associated with key, returning the new value.

Extracts the Vec<(T, i64)> from the map, consuming it.

Iterates over the contents of the map.

Clears the map.

The number of non-zero keys in the map.

True iff all keys have value zero.

Returns an element of the map, or None if it is empty.

Allocates a new empty CountMap.

Allocates a new CountMap with a single entry.

Drains self into other.

Performs a sequence of updates described by iterator.