Struct timely::progress::frontier::Antichain [] [src]

pub struct Antichain<T> { /* fields omitted */ }

A set of mutually incomparable elements.

Methods

impl<T: PartialOrder> Antichain<T>
[src]

Updates the Antichain if the element is not greater than or equal to some present element.

Returns true if element is added to the set

Creates a new empty Antichain.

Creates a new singleton Antichain.

Clears the contents of the antichain.

Returns true if any item in the antichain is strictly less than the argument.

Returns true if any item in the antichain is less than or equal to the argument.

Returns true if every element of other is greater or equal to some element of self.

Reveals the elements in the antichain.