Struct timely::progress::frontier::Antichain
[−]
[src]
pub struct Antichain<T> { /* fields omitted */ }A set of mutually incomparable elements.
Methods
impl<T: PartialOrder> Antichain<T>[src]
fn insert(&mut self, element: T) -> bool
Updates the Antichain if the element is not greater than or equal to some present element.
Returns true if element is added to the set
fn new() -> Antichain<T>
Creates a new empty Antichain.
fn from_elem(element: T) -> Antichain<T>
Creates a new singleton Antichain.
fn clear(&mut self)
Clears the contents of the antichain.
fn less_than(&self, time: &T) -> bool
Returns true if any item in the antichain is strictly less than the argument.
fn less_equal(&self, time: &T) -> bool
Returns true if any item in the antichain is less than or equal to the argument.
fn dominates(&self, other: &Antichain<T>) -> bool
Returns true if every element of other is greater or equal to some element of self.
fn elements(&self) -> &[T]
Reveals the elements in the antichain.