Struct timely::progress::nested::subgraph::Subgraph [] [src]

pub struct Subgraph<TOuter: Timestamp, TInner: Timestamp> {
    pub path: Vec<usize>,
    pub index: usize,
    // some fields omitted
}

A dataflow subgraph.

The subgraph type contains the infrastructure required to describe the topology of and track progress within a dataflow subgraph.

Fields

A sequence of integers uniquely identifying the subgraph.

The index assigned to the subgraph by its parent.

Methods

impl<TOuter: Timestamp, TInner: Timestamp> Subgraph<TOuter, TInner>
[src]

Allocates a new input to the subgraph and returns the assigned index.

Allocates a new output from the subgraph and returns the assigned index.

Introduces a dependence from the source to the target.

This method does not effect data movement, but rather reveals to the progress tracking infrastructure that messages produced by source should be expected to be consumed at target.

Creates a new Subgraph from a channel allocator and "descriptive" indices.

Allocates a new child identifier, for later use.

Adds a new child to the subgraph.