[−][src]Module timely::progress::timestamp
A partially ordered measure of progress at each timely dataflow location.
Traits
| PathSummary | A summary of how a timestamp advances along a timely dataflow path. |
| Refines | Conversion between pointstamp types. |
| Timestamp | A composite trait for types that serve as timestamps in timely dataflow. |