Trait timely::Push
[−]
pub trait Push<T> {
fn push(&mut self, element: &mut Option<T>);
fn send(&mut self, element: T) { ... }
fn done(&mut self) { ... }
}Pushing elements of type T.
Required Methods
fn push(&mut self, element: &mut Option<T>)
Pushes element and provides the opportunity to take ownership.
The value of element after the call may be changed. A change does not imply anything other
than that the implementor took resources associated with element and is returning other
resources.
Provided Methods
fn send(&mut self, element: T)
Pushes element and drops any resulting resources.
fn done(&mut self)
Pushes None, conventionally signalling a flush.
Implementors
impl<T, P> Push<T> for Box<P> where P: Push<T> + ?Sizedimpl<T: Clone + 'static, D: Abomonation + Clone + 'static> Push<(T, Content<D>)> for Tee<T, D>impl<T: Eq + Clone + 'static, D: Data + Abomonation, P: Push<(T, Content<D>)>, H: Fn(&D) -> u64> Push<(T, Content<D>)> for Exchange<T, D, P, H>impl<T, D, P: Push<(T, Content<D>)>> Push<(T, Content<D>)> for Counter<T, D, P> where T: Eq + Clone + 'staticimpl<T, D> Push<(T, Content<D>)> for Pusher<T, D>